@@ -1,7 +1,9 @@
 from __future__ import absolute_import
 
-from logging.handlers import TimedRotatingFileHandler
-from logging.handlers import MemoryHandler
+from loguru import logger
+from colorama import init as color_init
+from colorama import Fore, Style
+
 import multiprocessing.queues as mpq
 from multiprocessing import Process
 import multiprocessing
@@ -9,7 +11,6 @@ from datetime import datetime
 from copy import deepcopy
 from hashlib import md5
 import traceback as tb
-import logging
 import time
 import sys
 import os
@@ -19,8 +20,9 @@ import _pickle as pickle  # Python3
 # This works in Python3.x...
 from queue import Empty, Full
 
-from colorama import init as color_init
-from colorama import Fore, Style
+global PACKAGE_NAME
+PACKAGE_NAME = "Polymer"
+
 
 """ Polymer.py - Manage parallel tasks
      Copyright (C) 2021-2022 David Michael Pennington
@@ -28,36 +30,17 @@ from colorama import Fore, Style
      Copyright (C) 2019      David Michael Pennington at ThousandEyes
      Copyright (C) 2015-2019 David Michael Pennington at Samsung Data Services
 
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions are
-met:
+                    GNU GENERAL PUBLIC LICENSE
+                       Version 3, 29 June 2007
 
-1.  Redistributions of source code must retain the above copyright
-    notice, this list of conditions and the following disclaimer.
-
-2.  Redistributions in binary form must reproduce the above copyright
-    notice, this list of conditions and the following disclaimer in the
-    documentation and/or other materials provided with the distribution.
-
-3.  Neither the name of the copyright holder nor the names of its
-    contributors may be used to endorse or promote products derived from
-    this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
-TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
-PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
-LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ Copyright (C) 2007 Free Software Foundation, Inc. <http://fsf.org/>
+ Everyone is permitted to copy and distribute verbatim copies
+ of this license document, but changing it is not allowed.
 """
 
+
 class SharedCounter(object):
-    """ A synchronized shared counter.
+    """A synchronized shared counter.
     The locking done by multiprocessing.Value ensures that only a single
     process or thread may read or write the in-memory ctypes object. However,
     in order to do n += 1, Python performs a read followed by a write, so a
@@ -69,10 +52,10 @@ class SharedCounter(object):
     http://eli.thegreenplace.net/2012/01/04/shared-counter-with-pythons-multiprocessing/
     """
 
-    def __init__(self, n = 0):
-        self.count = multiprocessing.Value('i', n)
+    def __init__(self, n=0):
+        self.count = multiprocessing.Value("i", n)
 
-    def increment(self, n = 1):
+    def increment(self, n=1):
         """ Increment the counter by n (default = 1) """
         with self.count.get_lock():
             self.count.value += n
@@ -85,13 +68,13 @@ class SharedCounter(object):
 
 ################################################################################
 #
-#py23_mp_queue() is heavily based on the following github repo's commit...
-#http://github.com/vterron/lemon/commit/9ca6b4b1212228dbd4f69b88aaf88b12952d7d6f
-#Code license is GPLv3 according to github.com/vterron/lemon/setup.py
+# py3_mp_queue() is heavily based on the following github repo's commit...
+# http://github.com/vterron/lemon/commit/9ca6b4b1212228dbd4f69b88aaf88b12952d7d6f
+# Code license is GPLv3 according to github.com/vterron/lemon/setup.py
 #
 ################################################################################
-class py23_mp_queue(mpq.Queue):
-    """ A portable implementation of multiprocessing.Queue.
+class py3_mp_queue(mpq.Queue):
+    """A portable implementation of multiprocessing.Queue.
     Because of multithreading / multiprocessing semantics, Queue.qsize() may
     raise the NotImplementedError exception on Unix platforms like Mac OS X
     where sem_getvalue() is not implemented. This subclass addresses this
@@ -101,41 +84,35 @@ class py23_mp_queue(mpq.Queue):
     being raised, but also allows us to implement a reliable version of both
     qsize() and empty().
     """
+
     # NOTE This is the new implementation based on:
     #    https://github.com/vterron/lemon/blob/master/util/queue.py
 
     def __init__(self, *args, **kwargs):
         # Use ctx argument if using Python3.4+
-        try:
-            if sys.version_info >= (3, 4, 0):
-                super(py23_mp_queue, self).__init__(
-                    *args, ctx=multiprocessing.get_context("spawn"), **kwargs
-                )
-            else:
-                super(py23_mp_queue, self).__init__(*args, **kwargs)
+        super(py3_mp_queue, self).__init__(
+            *args, ctx=multiprocessing.get_context("spawn"), **kwargs
+        )
 
-        except Exception as ee:
-            raise(ee)
-
-        if not sys.platform=='darwin':
+        if not sys.platform == "darwin":
             self.size = SharedCounter(0)
 
     def put(self, *args, **kwargs):
         # Infinite recursion possible if we don't use super() here
         try:
-            super(py23_mp_queue, self).put(*args, **kwargs)
+            super(py3_mp_queue, self).put(*args, **kwargs)
 
         except Full:
             pass
 
-        if not sys.platform=='darwin':
+        if not sys.platform == "darwin":
             self.size.increment(1)
 
     def get(self, *args, **kwargs):
-        if not sys.platform=='darwin':
+        if not sys.platform == "darwin":
             self.size.increment(-1)
         try:
-            item = super(py23_mp_queue, self).get(*args, **kwargs)
+            item = super(py3_mp_queue, self).get(*args, **kwargs)
             return item
 
         except Empty:
@@ -143,14 +120,14 @@ class py23_mp_queue(mpq.Queue):
 
     def qsize(self):
         """ Reliable implementation of multiprocessing.Queue.qsize() """
-        if not sys.platform=='darwin':
+        if not sys.platform == "darwin":
             return self.size.value
         else:
             raise NotImplementedError
 
     def empty(self):
         """ Reliable implementation of multiprocessing.Queue.empty() """
-        if not sys.platform=='darwin':
+        if not sys.platform == "darwin":
             return not self.qsize()
         else:
             raise NotImplementedError
@@ -194,7 +171,7 @@ class Worker(object):
             self.task = None
 
     def done_q_send(self, msg_dict):
-        """Send message dicts through done_q, and throw explicit errors for 
+        """Send message dicts through done_q, and throw explicit errors for
         pickle problems"""
 
         # Check whether msg_dict can be pickled...
@@ -223,7 +200,10 @@ class Worker(object):
             )
             sys.stderr.write(
                 "{0}          {1}Pickling problems often come from open or hung TCP sockets{2}{3}".format(
-                    datetime.now(), Style.BRIGHT, Style.RESET_ALL, linesep,
+                    datetime.now(),
+                    Style.BRIGHT,
+                    Style.RESET_ALL,
+                    linesep,
                 )
             )
 
@@ -289,7 +269,7 @@ class Worker(object):
                 )
 
     def invalid_dict_pickle_keys(self, msg_dict):
-        """Return a list of keys that can't be pickled.  Return [] if 
+        """Return a list of keys that can't be pickled.  Return [] if
         there are no pickling problems with the values associated with the
         keys.  Return the list of keys, if there are problems."""
         no_pickle_keys = list()
@@ -383,11 +363,6 @@ class TaskMgrStats(object):
         self.exec_times = list()
         self.queue_times = list()
 
-    @property
-    def worker_pct_busy(self):
-        # return time_worked/total_work_time*100.0
-        return time_worked
-
     @property
     def time_delta(self):
         return time.time() - self.stats_start
@@ -435,7 +410,7 @@ class TaskMgrStats(object):
 
 
 class TaskMgr(object):
-    """Manage tasks to and from workers; maybe one day use zmq instead of 
+    """Manage tasks to and from workers; maybe one day use zmq instead of
     multiprocessing.Queue"""
 
     # http://www.jeffknupp.com/blog/2014/02/11/a-celerylike-python-task-queue-in-55-lines-of-code/
@@ -466,11 +441,11 @@ class TaskMgr(object):
         self.resubmit_on_error = resubmit_on_error
 
         # By default, Python3's multiprocessing.Queue doesn't implement qsize()
-        # NOTE:  OSX doesn't implement queue.qsize(), py23_mp_queue is a
+        # NOTE:  OSX doesn't implement queue.qsize(), py3_mp_queue is a
         #     workaround
         # AttributeError: 'module' object has no attribute 'get_context'
-        self.todo_q = py23_mp_queue()  # workers listen to todo_q (task queue)
-        self.done_q = py23_mp_queue()  # results queue
+        self.todo_q = py3_mp_queue()  # workers listen to todo_q (task queue)
+        self.done_q = py3_mp_queue()  # results queue
 
         self.worker_assignments = dict()  # key: w_id, value: worker Process objs
         self.results = dict()
@@ -486,29 +461,17 @@ class TaskMgr(object):
             self.supervise()  # hot_loops automatically supervise()
 
     def configure_logging(self):
+        logger.disable(PACKAGE_NAME)
         if self.log_level:
-            self.log = logging.getLogger(__name__)
-            fmtstr = "%(asctime)s.%(msecs).03d %(levelname)s %(message)s"
-            format = logging.Formatter(fmt=fmtstr, datefmt="%Y-%m-%d %H:%M:%S")
-            self.log.setLevel(logging.DEBUG)
 
             if self.log_path:
-                ## Rotate the logfile every day...
-                rotating_file = TimedRotatingFileHandler(
-                    self.log_path, when="D", interval=1, backupCount=5
-                )
-                rotating_file.setFormatter(format)
-                memory_log = logging.handlers.MemoryHandler(
-                    1024 * 10, logging.DEBUG, rotating_file
-                )
-                self.log.addHandler(memory_log)
+                logger.add(sink=self.log_path)
             if self.log_stdout:
-                console = logging.StreamHandler()
-                console.setFormatter(format)
-                self.log.addHandler(console)
+                logger.add(sink=sys.stdout)
 
             if not bool(self.log_path) and (not self.log_stdout):
                 self.log_level = 0
+            logger.enable(PACKAGE_NAME)
 
     def supervise(self):
         """If not in a hot_loop, call supervise() to start the tasks"""
@@ -522,7 +485,7 @@ class TaskMgr(object):
         hot_loop = self.hot_loop
         if self.log_level >= 2:
             logmsg = "TaskMgr.supervise() started {0} workers".format(self.worker_count)
-            self.log.info(logmsg)
+            logger.info(logmsg)
         self.workers = self.spawn_workers()
 
         ## Add work
@@ -532,12 +495,12 @@ class TaskMgr(object):
                 logmsg = "TaskMgr.supervise() received {0} tasks".format(
                     len(self.work_todo)
                 )
-                self.log.info(logmsg)
+                logger.info(logmsg)
             for task in self.work_todo:
                 self.num_tasks += 1
                 if self.log_level >= 2:
                     logmsg = "TaskMgr.supervise() queued task: {0}".format(task)
-                    self.log.info(logmsg)
+                    logger.info(logmsg)
                 self.queue_task(task)
 
         finished = False
@@ -557,9 +520,9 @@ class TaskMgr(object):
                     self.worker_assignments[w_id] = task
                     self.work_todo.remove(task)
                     if self.log_level >= 3:
-                        self.log.debug("r_msg: {0}".format(r_msg))
+                        logger.debug("r_msg: {0}".format(r_msg))
                     if self.log_level >= 3:
-                        self.log.debug("w_id={0} received task={1}".format(w_id, task))
+                        logger.debug("w_id={0} received task={1}".format(w_id, task))
                 elif state == "__FINISHED__":
                     now = time.time()
                     task_exec_time = task.task_stop - task.task_start
@@ -568,21 +531,23 @@ class TaskMgr(object):
                     stats.queue_times.append(task_queue_time)
 
                     if self.log_level >= 1:
-                        self.log.info(
+                        logger.info(
                             "TaskMgr.work_todo: {0} tasks left".format(
                                 len(self.work_todo)
                             )
                         )
                     if self.log_level >= 3:
-                        self.log.debug("TaskMgr.work_todo: {0}".format(self.work_todo))
-                        self.log.debug("r_msg: {0}".format(r_msg))
+                        logger.debug("TaskMgr.work_todo: {0}".format(self.work_todo))
+                        logger.debug("r_msg: {0}".format(r_msg))
 
                     if not hot_loop:
                         self.retval.add(task)  # Add result to retval
                         self.worker_assignments.pop(w_id)  # Delete the key
                         finished = self.is_finished()
                     else:
-                        self.controller.from_taskmgr_q.put(task)  # Send to the controller
+                        self.controller.from_taskmgr_q.put(
+                            task
+                        )  # Send to the controller
                         self.worker_assignments.pop(w_id)  # Delete the key
 
                 elif state == "__ERROR__":
@@ -594,15 +559,15 @@ class TaskMgr(object):
                     stats.queue_times.append(task_queue_time)
 
                     if self.log_level >= 1:
-                        self.log.error("r_msg: {0}".format(r_msg))
-                        self.log.error("".join(r_msg.get("error")))
-                        self.log.debug(
+                        logger.error("r_msg: {0}".format(r_msg))
+                        logger.error("".join(r_msg.get("error")))
+                        logger.debug(
                             "TaskMgr.work_todo: {0} tasks left".format(
                                 len(self.work_todo)
                             )
                         )
                     if self.log_level >= 3:
-                        self.log.debug("TaskMgr.work_todo: {0}".format(self.work_todo))
+                        logger.debug("TaskMgr.work_todo: {0}".format(self.work_todo))
 
                     if not hot_loop:
                         if not self.resubmit_on_error:
@@ -630,7 +595,7 @@ class TaskMgr(object):
 
             if stats.log_time:
                 if self.log_level >= 0:
-                    self.log.info(stats.log_message)
+                    logger.info(stats.log_message)
 
             # Adaptive loop delay unless on Mac OSX... OSX delay is constant...
             delay = self.calc_wait_time(stats.exec_times)
@@ -646,14 +611,14 @@ class TaskMgr(object):
 
             ## Log a final stats summary...
             if self.log_level > 0:
-                self.log.info(stats.log_message)
+                logger.info(stats.log_message)
             return self.retval
 
     def calc_wait_time(self, exec_times):
         num_samples = float(len(exec_times))
 
         # Work around Mac OSX problematic queue.qsize() implementation...
-        if sys.platform=='darwin':
+        if sys.platform == "darwin":
             wait_time = 0.00001  # 10us delay to avoid worker / done_q race
 
         elif num_samples > 0.0:
@@ -707,10 +672,10 @@ class TaskMgr(object):
             if not p.is_alive():
                 # Queue the task for another worker, if required...
                 if self.log_level >= 2:
-                    self.log.info("Worker w_id {0} died".format(w_id))
+                    logger.info("Worker w_id {0} died".format(w_id))
                 task = self.worker_assignments.get(w_id, {})
                 if self.log_level >= 2 and task != {}:
-                    self.log.info(
+                    logger.info(
                         "Dead worker w_id {0} was assigned task - {1}".format(
                             w_id, task
                         )
@@ -722,14 +687,14 @@ class TaskMgr(object):
                         self.work_todo.append(task)
                         self.queue_task(task)
                         if self.log_level >= 2:
-                            self.log.info("Resubmitting task - {0}".format(task))
+                            logger.info("Resubmitting task - {0}".format(task))
                         error_suffix = " with task={1}".format(task)
                 if self.log_level >= 1:
-                    self.log.debug(
+                    logger.debug(
                         "TaskMgr.work_todo: {0} tasks left".format(len(self.work_todo))
                     )
                 if self.log_level >= 2:
-                    self.log.info(
+                    logger.info(
                         "Respawning worker - w_id={0}{1}".format(w_id, error_suffix)
                     )
                 self.workers[w_id] = Process(
@@ -839,7 +804,7 @@ class TaskMgr(object):
                 )
 
     def invalid_dict_pickle_keys(self, msg_dict):
-        """Return a list of keys that can't be pickled.  Return [] if 
+        """Return a list of keys that can't be pickled.  Return [] if
         there are no pickling problems with the values associated with the
         keys.  Return the list of keys, if there are problems."""
         no_pickle_keys = list()
@@ -874,5 +839,5 @@ class ControllerQueue(object):
 
     def __init__(self):
         ## to and from are with respect to the (client) controller object
-        self.from_taskmgr_q = py23_mp_queue()  # sent to the controller from TaskMgr
-        self.to_taskmgr_q = py23_mp_queue()  # sent from the controller to TaskMgr
+        self.from_taskmgr_q = py3_mp_queue()  # sent to the controller from TaskMgr
+        self.to_taskmgr_q = py3_mp_queue()  # sent from the controller to TaskMgr
